package de.thk.gm.websocketsdemo.handlers

import de.thk.gm.websocketsdemo.models.ChatMessage
import de.thk.gm.websocketsdemo.services.ChatMessageService
import de.thk.gm.websocketsdemo.services.ChatRoomsService
import org.json.JSONObject
import org.springframework.web.socket.CloseStatus
import org.springframework.web.socket.TextMessage
import org.springframework.web.socket.WebSocketSession
import org.springframework.web.socket.handler.TextWebSocketHandler
import org.springframework.web.util.UriComponents
import org.springframework.web.util.UriComponentsBuilder
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

class ChatRoomsHandler (private val chatRoomsService: ChatRoomsService, private val chatMessageService: ChatMessageService) : TextWebSocketHandler() {
    private val hashMapOfSessions : HashMap<String, ArrayList<WebSocketSession>> = HashMap()
    override fun afterConnectionEstablished(session: WebSocketSession) {
        var uri : UriComponents = UriComponentsBuilder.fromUri(session.uri!!).build()
        var id = uri.queryParams.getFirst("id")
        if(id != null) {
            var sessions = hashMapOfSessions[id]
            if(sessions == null) {
                sessions = ArrayList()
            }
            sessions.add(session)
            hashMapOfSessions[id] = sessions
        }
    }

    override fun handleTextMessage(session: WebSocketSession, message: TextMessage) {
        var uri : UriComponents = UriComponentsBuilder.fromUri(session.uri!!).build()
        var id  = uri.queryParams.getFirst("id")
        var sessions = hashMapOfSessions[id]
        val chatMessageJSONObject = JSONObject(message.payload)

        val chatRoom = chatRoomsService.getChatRoomById(UUID.fromString(id));
        val chatMessage = ChatMessage()
        chatMessage.chatRoom = chatRoom
        chatMessage.text = chatMessageJSONObject.getString("text")
        chatMessage.username = chatMessageJSONObject.getString("username")
        chatMessageService.save(chatMessage)

        if(sessions != null) {
            for (chatSession in sessions) {
                chatSession.sendMessage(message)
            }
        }
    }

    override fun afterConnectionClosed(session: WebSocketSession, status: CloseStatus) {
        var uri : UriComponents = UriComponentsBuilder.fromUri(session.uri!!).build()
        var id = uri.queryParams.getFirst("id")
        var sessions = hashMapOfSessions[id]
        sessions?.remove(session)
    }

}